#!/usr/bin/perl

#########################################################
# Settings #
############

# Pass message without checking if error appeared while message scanning
my $ALLOW_IF_ERROR="No";

# Name (domain or IP) of machine where DrWeb daemon runs
my $SERVER='127.0.0.1';

# DrWeb daemon port on server
my $PORT='3000';

# E-mail which daemon writes reports from
my $DAEMON_EMAIL="DrWeb-DAEMON\@your.domain";

# Postmaster's e-mail
my $POSTMASTER_EMAIL="postmaster\@your.domain";

# E-mail for alerts and problems reports (can be local)
my $ADMIN_EMAIL="sergey";

# Directory for save message before check
my $TMP_DIR="/var/spool/drweb";

# Directory for infected messages
my $INFECTED_DIR="/opt/drweb/infected.!!!";

# Directory where DrWeb is installed
my $DRWEB_DIR="/opt/drweb";

# Command for postfix wrapper for sendmail
my $MAIL_CMD="/usr/sbin/sendmail";

# Enable (Yes) or disable (Other) logging
my $LOG_ENABLE="No";

# Path to logfile 
my $LOG_FILE="/opt/drweb/postfix.log";

#########################################################
# Subroutine
sub execute
{
    my ($Cmd,$Code,$Msg) = @_;

    $$Msg = `$Cmd`;
    $$Code = ($? >> 8);
    return $$Code;
}

# Variables and constants

my $SCANER_CMD="$DRWEB_DIR/clients/drwebdc/drwebdc -p$PORT -n$SERVER -rv -f";

my $EX_OK=0;
my $EX_TEMPFAIL=75;

my $Status;
my $StatusMsg="";
my $Line;
my $ExitStatus;

#########################################################
# Start code here
if ($LOG_ENABLE eq "Yes")
{
    open (LOGFILE, ">>$LOG_FILE");
}
else
{
    open (LOGFILE, "/dev/null");
}

# Extract arguments
my $time=localtime;
my $Sender=shift(@ARGV);
my @Recipient=@ARGV;
print LOGFILE "$time - process message: from=$Sender to=@Recipient\n";

# Flush arguments
$#ARGV=0;
pop @ARGV;

# create temp file
my $TEMPLATE="${TMP_DIR}/msg.XXXXXX";
execute( "mktemp -q $TEMPLATE", \$Status, \$TMP_FILE );
if ($Status)
{
    print LOGFILE "$time - cannot create tempfile in ${TMP_DIR}.\n";
    close (LOGFILE);
    # Cannot create tempfile
    if ($ALLOW_IF_ERROR eq "Yes")
    {
        # Problem with daemon or client
        open (OUTFILE,"| $MAIL_CMD -bm -f $DAEMON_EMAIL");
        print OUTFILE "From: Antivirus Service <$DAEMON_EMAIL>\n";
        print OUTFILE "To: Postmaster <$ADMIN_EMAIL>\n";
        print OUTFILE "Subject: Antivirus Software Fail\n";
        print OUTFILE "Dear Postmaster,\n";
        print OUTFILE "Message sended by from $Sender to @Recipient failed to scan.\n";
        print OUTFILE "Diagnostic: cannot create tempfile for messages: error code - $Status\n";
        print OUTFILE "---\n";
        print OUTFILE "\t\tAntivirus service provided by Dr.Web Daemon\n";
        close (OUTFILE);        
        exit ($EX_OK);
    }
    else
    {
        if ($ALLOW_IF_ERROR == "Yes")
        {
            $ExitStatus = $EX_OK;
        }
        else
        {
            $ExitStatus = $EX_TEMPFAIL;
        }
        exit ($ExitStatus);
    }
}
# Delete \n at end of filename
$TMP_FILE =~ s/\n$//;

open (OUTFILE,">$TMP_FILE");
while ($Line = <>)
{
    print OUTFILE $Line;
}
close (OUTFILE);

execute ("$SCANER_CMD $TMP_FILE",\$Status,\$StatusMsg);
{
    if ($Status == 0)
    {
        #virus not found        
        print LOGFILE "$time - message $TMP_FILE is ok.\n";
        print LOGFILE "$time - exec: $MAIL_CMD -bm -f $Sender @Recipient < $TMP_FILE\n";
        system("$MAIL_CMD -bm -f $Sender @Recipient < $TMP_FILE");
        $ExitStatus = $EX_OK;
    }
    else
    {
        if (($Status == 32) || ($Status == 64) || ($Status == 128))
        {
            print LOGFILE "$time - message $TMP_FILE infected.\n";
            #virus found
            my @Lines = split(/\n/, $StatusMsg);
            my @VirusInfo;
            foreach $Line (@Lines) 
            {
                if ($Line =~ /^infected/) 
                {
                    push(@VirusInfo,$Line);
                }
            }

            #mail to postmater
            open (OUTFILE,"| $MAIL_CMD -bm -f $DAEMON_EMAIL");
            print OUTFILE "From: Antivirus Service <$DAEMON_EMAIL>\n";
            print OUTFILE "To: Postmaster <$ADMIN_EMAIL>\n";
            print OUTFILE "Subject: VIRUS ALERT !\n\n";
            print OUTFILE "Dear Postmaster,\n";
            print OUTFILE "Message sended by $Sender to @Recipient";
            print OUTFILE " :\n";
            foreach $Virus (@VirusInfo)
            {
                print OUTFILE " \t$Virus\n";
            }
            print OUTFILE "---\n";
            print OUTFILE "\t\tAntivirus service provided by Dr.Web Daemon\n";
            close (OUTFILE);

            #mail to SENDER if it is known

            if ($Sender !~ /^UNKNOWN$/)
            {
                open (OUTFILE,"| ${MAIL_CMD} -bm -f $DAEMON_EMAIL");
                print OUTFILE "From: Antivirus Service <$DAEMON_EMAIL>\n";
                print OUTFILE "To: <${Sender}>\n";
                print OUTFILE "Subject: Mail delivery failed (virus detected)\n\n";
                print OUTFILE "Your message sended to $Recipient fail to delivery.\n";
                print OUTFILE "Reason: viruses detected.\n";
                print OUTFILE "Diagnostic:\n";
                foreach $Virus (@VirusInfo)
                {
                    print OUTFILE " \t$Virus\n";
                }
                print OUTFILE "Original message stored in archive.\n";
                print OUTFILE "To get message you should write to address:\n$POSTMASTER_EMAIL\n";
                print OUTFILE " \n";
                print OUTFILE "---\n";
                print OUTFILE "\t\tAntivirus service provided by Dr.Web Daemon\n";
                close (OUTFILE);
            }

            # Move infected msg to archive
            system ("/bin/mv -f $TMP_FILE $INFECTED_DIR");
            close (LOGFILE);
            exit ($EX_OK);
        }
        else
        {
            print LOGFILE "$time - message $TMP_FILE was not checked: errors durig processing.\n";
            # Problem with daemon or client
            open (OUTFILE,"| $MAIL_CMD -bm -f $DAEMON_EMAIL");
            print OUTFILE "From: Antivirus Service <$DAEMON_EMAIL>\n";
            print OUTFILE "To: Postmaster <$ADMIN_EMAIL>\n";
            print OUTFILE "Subject: Antivirus Software Fail\n";
            print OUTFILE "Dear Postmaster,\n";
            print OUTFILE "Message sended by from $Sender to $Recipient failed to scan.\n";
            print OUTFILE "Diagnostic: error code - $Status\n";
            print OUTFILE "---\n";
            print OUTFILE "\t\tAntivirus service provided by Dr.Web Daemon\n";
            close (OUTFILE);

            if( $ALLOW_IF_ERROR == "Yes" )
            {
                system("$MAIL_CMD -bm -f $Sender @Recipient < $TMP_FILE");
                $ExitStatus = $EX_OK;
            }
            else
            {
                $ExitStatus = $EX_TEMPFAIL;
            }
        }
    }
}
# Exit, delete temp-file before
close (LOGFILE);
unlink ($TMP_FILE);
exit ($ExitStatus);
