#!/bin/bash

#########################################################
# Settings #
############

# if error allow or not to pass message without checking
ALLOW_IF_ERROR="No"

# Name (domain or IP) machine there DrWeb daemon running
SERVER="localhost"

# DrWeb daemon port on server 
PORT="3000"

# E-Mail from that daemon writes reports
DAEMON_EMAIL="DrWeb-DAEMON@your.domain"

# E-Mail that placed to reports 
POSTMASTER_EMAIL="postmaster@your.domain"

# Mail address for alerts and problems reports (can be local)
ADMIN_EMAIL="root"

# Directory for save message before check
TMP_DIR=/var/spool/drweb

# Directory for infected messages
INFECTED_DIR=/opt/drweb/infected.!!!

# Directory where DrWeb installed
DRWEB_DIR=/opt/drweb

# Command for postfix wrapper for sendmail
SENDMAIL_CMD="/usr/sbin/sendmail"

# Enable (Yes) or disable (Other) logging
LOG_ENABLE="No"

# Path to logfile 
LOG_FILE=/opt/drweb/postfix.log

#########################################################
# Variables and constants

# Command for client to daemon
SCANNER_CMD="$DRWEB_DIR/clients/drwebdc/drwebdc -n"$SERVER" -p"$PORT" -q -f"

# Exit codes from <sysexits.h>
EX_OK=0
EX_TEMPFAIL=75
EX_UNAVAILABLE=69

##############
# Start code #
##############

# Check environment
if [ ! -d $TMP_DIR ] ; then
    if [ $LOG_ENABLE = "Yes" ] ; then
        echo -n `date` >> $LOG_FILE
        echo " - $TMP_DIR does not exist" >> $LOG_FILE
    fi
    if [ $ALLOW_IF_ERROR = "Yes" ] ; then
        exit $EX_OK
    else
        exit $EX_TEMPFAIL   
    fi	
fi    

# Parse arguments
ARGV=( $* )
SENDER=$1
for (( num = 1; $num < $#; num = `expr $num + 1` ));
do
    i=`expr $num - 1`
    RECEIVERS[$i]=${ARGV[$num]}
done

if [ $LOG_ENABLE = "Yes" ] ; then
    echo -n `date` >> $LOG_FILE
    echo " - process message: from=$SENDER to=${RECEIVERS[*]}" >> $LOG_FILE
fi    

# Create tempfile
TEMP_FILE=`mktemp -q $TMP_DIR/msg.XXXXXX`

# Clean up when done or when aborting.
trap "rm -f $TEMP_FILE" 0 1 2 3 15

# Save to tempfile.
cat > $TEMP_FILE 
if [ $? -ne 0 ] ; then
    if [ $LOG_ENABLE = "Yes" ] ; then
        echo -n `date` >> $LOG_FILE
        echo " - cannot save mail to file" >> $LOG_FILE
    fi
    if [ $ALLOW_IF_ERROR = "Yes" ] ; then
        exit $EX_OK
    else
        exit $EX_TEMPFAIL   
    fi	
fi    

# Call Dr.Web.
$SCANNER_CMD $TEMP_FILE
EXIT_STATUS=$?
if [ $EXIT_STATUS -eq 0 ]
then
    if [ $LOG_ENABLE = "Yes" ] ; then
        echo -n `date` >> $LOG_FILE        
        echo " - message $TEMP_FILE is ok."  >> $LOG_FILE
        echo -n `date` >> $LOG_FILE        
        echo " - exec: $SENDMAIL_CMD -bm -f $SENDER ${RECEIVERS[*]} < $TEMP_FILE" >> $LOG_FILE
    fi
    # Send normal message
    $SENDMAIL_CMD -bm -f $SENDER ${RECEIVERS[*]} < $TEMP_FILE
    if [ $? -ne 0 ] ; then
        if [ $LOG_ENABLE = "Yes" ] ; then
            echo -n `date` >> $LOG_FILE
            echo " - cannot reinject message $TEMP_FILE to mail system" >> $LOG_FILE
        fi
        rm -f $TEMP_FILE    
        if [ $ALLOW_IF_ERROR = "Yes" ] ; then
            exit $EX_OK
        else
            exit $EX_TEMPFAIL   
        fi	
    else
        rm -f $TEMP_FILE    
        exit $EX_OK
    fi    
elif [ $EXIT_STATUS -eq 32 -o $EXIT_STATUS -eq 64 -o $EXIT_STATUS -eq 128 ] 
then
    if [ $LOG_ENABLE = "Yes" ] ; then
        echo -n `date` >> $LOG_FILE        
        echo " - message $TEMP_FILE infected."  >> $LOG_FILE
    fi
    
    # Move infected message to archive
    mv -f $TEMP_FILE $INFECTED_DIR    
    if [ $? -ne 0 ] ; then
        if [ $LOG_ENABLE = "Yes" ] ; then
            echo -n `date` >> $LOG_FILE
            echo " - cannot move infected message $TEMP_FILE to $INFECTED_DIR" >> $LOG_FILE
        fi
    fi    
    
    # Generate message to sender
    TEMP_FILE=`mktemp -q $TMP_DIR/drweb.XXXXXX`
    printf "From: "$DAEMON_EMAIL"\nTo: "$SENDER"\nSubject: Mail delivery failed (virus detected)\nYour message sended to "${RECEIVERS[*]}" failed to delivery.\nReason: virus detected.\nOriginal message stored in archive.\nTo receive original, please send request on "$POSTMASTER_EMAIL"\n---\n   Antivirus service provided by Dr.Web Daemon\n.\n" >> $TEMP_FILE
	    
    # Send virus alert
    $SENDMAIL_CMD -bm -f $DAEMON_EMAIL < $TEMP_FILE
    if [ $? -ne 0 ] ; then
        if [ $LOG_ENABLE = "Yes" ] ; then
            echo -n `date` >> $LOG_FILE
            echo " - cannot send alert to $SENDER" >> $LOG_FILE
        fi
    fi    
    rm -f $TEMP_FILE

    # Generate message to admin
    TEMP_FILE=`mktemp -q $TMP_DIR/drweb.XXXXXX`
    
    printf "From: "$DAEMON_EMAIL"\nTo: "$ADMIN_EMAIL"\nSubject: Mail delivery failed (virus detected)\nDear Postmaster,\nMessage sended by "$SENDER" \nto "${RECEIVERS[*]}" failed to delivery.\nReason: virus detected.\n---\n   Antivirus service provided by Dr.Web Daemon\n.\n" >> $TEMP_FILE 
	    
    # Send report to admin
    $SENDMAIL_CMD -bm -f $DAEMON_EMAIL < $TEMP_FILE
    if [ $? -ne 0 ] ; then
        if [ $LOG_ENABLE = "Yes" ] ; then
            echo -n `date` >> $LOG_FILE
            echo " - cannot send alert to $ADMIN_EMAIL" >> $LOG_FILE
        fi
    fi    
    rm -f $TEMP_FILE
    
    exit $EX_OK
else
    if [ $LOG_ENABLE = "Yes" ] ; then
        echo -n `date` >> $LOG_FILE        
        echo " - message $TEMP_FILE was not checked."  >> $LOG_FILE
    fi
    # Any problems with daemon or client
    if [ $ALLOW_IF_ERROR = "Yes" ] ; then
        $SENDMAIL_CMD -bm -f $SENDER < $TEMP_FILE
        if [ $? -ne 0 ] ; then
            if [ $LOG_ENABLE = "Yes" ] ; then
                echo -n `date` >> $LOG_FILE
                echo " - cannot reinject message $TEMP_FILE to mail system" >> $LOG_FILE
            fi
        fi
    fi
    rm -f $TEMP_FILE
    
    # Generate report
    TEMP_FILE=`mktemp -q $TMP_DIR/drweb.XXXXXX`
    
    if [ $EXIT_STATUS -le 65535 ] ; then
        printf "From: "$DAEMON_EMAIL"\nTo: "$ADMIN_EMAIL"\nSubject: Antivirus Software Fail\nHello,\nMessage sended by from "$SENDER" \nto "${RECEIVERS[*]}" failed to scan.\nDiagnostic: possible problems with daemon (error code "$EXIT_STATUS")\n---\n   Antivirus service provided by Dr.Web Daemon\n.\n" >> $TEMP_FILE
    else
        printf "From: "$DAEMON_EMAIL"\nTo: "$ADMIN_EMAIL"\nSubject: Antivirus Software Fail\nHello,\nMessage sended by from "$SENDER" \nto "${RECEIVERS[*]}" failed to scan.\nDiagnostic: possible problems with client (error code "$EXIT_STATUS")\n---\n   Antivirus service provided by Dr.Web Daemon\n.\n" >> $TEMP_FILE
    fi	
	    
    # Send report to admin
    $SENDMAIL_CMD -bm -f $DAEMON_EMAIL < $TEMP_FILE
    if [ $? -ne 0 ] ; then
        if [ $LOG_ENABLE = "Yes" ] ; then
            echo -n `date` >> $LOG_FILE
            echo " - cannot send alert to $ADMIN_EMAIL" >> $LOG_FILE
        fi
    fi        
    rm -f $TEMP_FILE
    
    if [ $ALLOW_IF_ERROR = "Yes" ] ; then
        exit $EX_OK
    else
        exit $EX_TEMPFAIL   
    fi	
fi

